package com.sf.vertx.init;

import com.alibaba.fastjson2.JSONObject;
import com.sf.vertx.enums.GatewayError;
import io.vertx.config.ConfigRetriever;
import io.vertx.config.ConfigRetrieverOptions;
import io.vertx.config.ConfigStoreOptions;
import io.vertx.core.AbstractVerticle;
import io.vertx.core.AsyncResult;
import io.vertx.core.Future;
import io.vertx.core.Handler;
import io.vertx.core.http.HttpHeaders;
import io.vertx.core.http.HttpServerOptions;
import io.vertx.core.http.HttpServerResponse;
import io.vertx.core.json.JsonObject;
import io.vertx.core.net.JksOptions;
import io.vertx.ext.web.Route;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.BodyHandler;
import io.vertx.ext.web.handler.CorsHandler;
import io.vertx.ext.web.handler.SessionHandler;
import io.vertx.ext.web.sstore.ClusteredSessionStore;
import io.vertx.ext.web.sstore.LocalSessionStore;
import io.vertx.ext.web.sstore.SessionStore;
import lombok.extern.slf4j.Slf4j;

import java.text.MessageFormat;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * 网关配置同步与管理
 */
@Slf4j
public class GatewayConfigVerticle extends AbstractVerticle {

    /**
     * HTTP的路由服务
     */
    private static Router httpRouter = null;

    private static VertxProperties vertxProperties;

    /**
     * 全局IP黑名单
     */
    private Set<String> blackIpSet = new LinkedHashSet<>();

    public void start() {
        ConfigStoreOptions store = new ConfigStoreOptions()
                .setType("file")
                .setFormat("yaml")
                .setConfig(new JsonObject()
                        .put("path", "application.yaml")
                );

        ConfigRetriever retriever = ConfigRetriever.create(vertx,
                new ConfigRetrieverOptions().addStore(store));
        retriever.getConfig(json -> {
            if (json.succeeded()) {
                doStart(json);
            } else {
                stop();
            }
        });


    }


    private void doStart(AsyncResult<JsonObject> json) {
        JsonObject config = json.result();
        log.info("配置读取成功");
        // 读取vertx配置
        JsonObject vertxConfig = config.getJsonObject("vertx");
        // 读取redis配置
        JsonObject redisConfig = config.getJsonObject("redis");

        //先初始化再发布Http服务
        vertx.executeBlocking(() -> {
            //顺序不能乱
            try {
                //初始化Redis
                RedisManager.initRedisConfig(vertx, redisConfig, redisConnectionAsyncResult -> {
                    if (redisConnectionAsyncResult.succeeded()) {
                        log.info("GatewayConfig redis初始化成功");
                    } else {
                        log.error("GatewayConfig redis初始化失败", redisConnectionAsyncResult.cause());
                        stop();
                    }
                });
            } catch (Exception e) {
                log.error("GatewayConfig redis初始化失败:", e);
                stop();
            }
            return true;
        }).onComplete(ar -> {
            // 启动服务
            if (ar.succeeded()) {
                vertxProperties = vertxConfig.mapTo(VertxProperties.class);
                blackIpSet = new LinkedHashSet<>(List.of(vertxProperties.getBlackIP()));
                createHttpServer(createServerAR ->{
                    if (createServerAR.succeeded()) {
                        log.info("GatewayConfig 启动服务成功");
                        // 添加路由
                        Route addAppConfigRoute = httpRouter.post("/vertx/app/config");
                        addAppConfigRoute.handler(BodyHandler.create(false))
                                .handler(ctx ->{
                            String s = ctx.body().asString();
                            log.info(s);
                            JSONObject jsonre = new JSONObject();
                            jsonre.put("code", 200);
                            jsonre.put("msg", "success");
                            ctx.response()
                                    .putHeader("Content-Type", "application/json")
                                    .end(jsonre.toJSONString());
                        });

                    } else {
                        log.error("GatewayConfig 启动服务失败", createServerAR.cause());
                        stop();
                    }
                });
            } else {
                log.error("GatewayConfig 启动服务失败:", ar.cause());
                stop();
            }
        });
    }


    public void stop() {
        log.info("close GatewayConfigVerticle !");
        vertx.close();
    }

    /**
     * 创建http服务器
     *
     * @param createHttp
     */
    public void createHttpServer(Handler<AsyncResult<Void>> createHttp) {
        // 所有ip都能访问
        HttpServerOptions httpServerOptions = new HttpServerOptions().setHost("0.0.0.0");
        if (vertxProperties.isSSL()) {
            httpServerOptions.setSsl(true)
                    .setKeyStoreOptions(new JksOptions().setPassword("changeit").setPath("keystore.jks"));
        }
        httpRouter = Router.router(vertx);
        httpRouter.route().handler(this::filterBlackIP);
        httpRouter.route().handler(CorsHandler.create().addRelativeOrigin(".*"));
        // 创建http服务器
        vertx.createHttpServer(httpServerOptions)
                .requestHandler(httpRouter)
                .listen(vertxProperties.getConfigPort(), res -> {
            if (res.succeeded()) {
                log.info("GatewayConfigServer Running on port {} by HTTP", vertxProperties.getConfigPort());
                createHttp.handle(Future.succeededFuture());
            } else {
                log.error("create HTTP Server (GatewayConfigServer) failed : GatewayConfigServer Running on port {} by HTTP", vertxProperties.getConfigPort());
                createHttp.handle(Future.failedFuture(res.cause()));
            }
        });
    }

    /**
     * 过滤黑名单
     *
     * @param rct
     */
    public void filterBlackIP(RoutingContext rct) {

        String host = rct.request().remoteAddress().host();
        if (blackIpSet.contains(host)) {
            HttpServerResponse response = rct.response();
                response.putHeader(HttpHeaders.CONTENT_TYPE, "text/html");
            response.setStatusCode(GatewayError.FORBIDDEN.getCode());
            response.setStatusMessage(GatewayError.FORBIDDEN.getReasonPhrase());
            response.end("<html>" +
                    "<body><h1>you can't access this service</h1></body>" +
                    "</html>");
        } else {
            rct.next();
        }
    }
}